@@ -181,7 +181,9 @@ gnetwork_connection_open (GNetworkConnection * connection)
 
   g_return_if_fail (iface->open != NULL);
 
+  g_object_ref (connection);
   (*iface->open) (connection);
+  g_object_unref (connection);
 }
 
 
@@ -204,7 +206,9 @@ gnetwork_connection_close (GNetworkConnection * connection)
 
   g_return_if_fail (iface->close != NULL);
 
+  g_object_ref (connection);
   (*iface->close) (connection);
+  g_object_unref (connection);
 }
 
 
@@ -239,7 +243,9 @@ gnetwork_connection_send (GNetworkConnection * connection, gconstpointer data, g
       for (length = 0; *(G_UCHAR (data) + length) != 0; length++);
     }
 
+  g_object_ref (connection);
   (*iface->send) (connection, data, (gulong) length);
+  g_object_unref (connection);
 }
 
 
@@ -262,7 +268,9 @@ gnetwork_connection_received (GNetworkConnection * connection, gconstpointer dat
   g_return_if_fail (data != NULL);
   g_return_if_fail (length > 0);
 
+  g_object_ref (connection);
   g_signal_emit (connection, signals[RECEIVED], 0, data, length);
+  g_object_unref (connection);
 }
 
 
@@ -285,7 +293,9 @@ gnetwork_connection_sent (GNetworkConnection * connection, gconstpointer data, g
   g_return_if_fail (data != NULL);
   g_return_if_fail (length > 0);
 
+  g_object_ref (connection);
   g_signal_emit (connection, signals[SENT], 0, data, length);
+  g_object_unref (connection);
 }
 
 
@@ -305,7 +315,9 @@ gnetwork_connection_error (GNetworkConnection * connection, const GError * error
   g_return_if_fail (GNETWORK_IS_CONNECTION (connection));
   g_return_if_fail (error != NULL);
 
+  g_object_ref (connection);
   g_signal_emit (connection, signals[ERROR], error->domain, error);
+  g_object_unref (connection);
 }
 
 
