@@ -98,9 +98,6 @@ get_ssl_cert_error_from_error_num (gint error_num)
   /* FIXME: How/should we handle error_num < 0? */
   if (error > 0)
     {
-      if (error_num & GNUTLS_CERT_NOT_TRUSTED)
-	error |= GNETWORK_SSL_CERT_ERROR_NOT_TRUSTED;
-
       if (error_num & GNUTLS_CERT_INVALID)
 	error |= GNETWORK_SSL_CERT_ERROR_INVALID;
 
