@@ -236,7 +236,7 @@ gnetwork_tcp_connection_close (GNetworkTcpConnection * connection)
   g_object_notify (object, "tcp-status");
   g_object_notify (object, "status");
   g_object_thaw_notify (object);
-  
+
   if (connection->_priv->dns_handle != GNETWORK_DNS_HANDLE_INVALID)
     {
       gnetwork_dns_cancel (connection->_priv->dns_handle);
@@ -278,7 +278,7 @@ gnetwork_tcp_connection_close (GNetworkTcpConnection * connection)
       g_free (item->data);
       g_free (item);
 
-      connection->_priv->buffer = g_slist_remove_link (connection->_priv->buffer,
+      connection->_priv->buffer = g_slist_delete_link (connection->_priv->buffer,
 						       connection->_priv->buffer);
     }
 
@@ -314,11 +314,11 @@ io_channel_handler (GIOChannel * channel, GIOCondition cond, GNetworkTcpConnecti
   if (cond & (G_IO_IN | G_IO_PRI))
     {
       GIOStatus status;
-      guchar *buffer;
+      guchar buffer[connection->_priv->buffer_size + 1];
       gsize bytes_read;
       GError *error = NULL;
 
-      buffer = g_new0 (guchar, connection->_priv->buffer_size + 1);
+      memset (buffer, 0, G_N_ELEMENTS (buffer));
 
       status = g_io_channel_read_chars (channel, buffer, connection->_priv->buffer_size,
 					&bytes_read, &error);
@@ -356,8 +356,6 @@ io_channel_handler (GIOChannel * channel, GIOCondition cond, GNetworkTcpConnecti
 	  g_assert_not_reached ();
 	  break;
 	}
-
-      g_free (buffer);
     }
 
   /* Write */
@@ -396,14 +394,15 @@ io_channel_handler (GIOChannel * channel, GIOCondition cond, GNetworkTcpConnecti
 		    {
 		      BufferItem *new_item = g_new0 (BufferItem, 1);
 
-		      new_item->data = g_new (guchar, item->length - bytes_sent + 1);
-		      memcpy (new_item->data, G_UCHAR (item->data) + bytes_sent, new_item->length);
 		      new_item->length = item->length - bytes_sent;
+		      new_item->data = g_new (guchar, new_item->length + 1);
+		      memcpy (new_item->data, G_UCHAR (item->data) + bytes_sent,
+			      new_item->length);
 		      connection->_priv->buffer->data = new_item;
 		    }
 		  else
 		    {
-		      connection->_priv->buffer = g_slist_remove_link (connection->_priv->buffer,
+		      connection->_priv->buffer = g_slist_delete_link (connection->_priv->buffer,
 								       connection->_priv->buffer);
 		    }
 		}
@@ -433,12 +432,18 @@ io_channel_handler (GIOChannel * channel, GIOCondition cond, GNetworkTcpConnecti
       /* We've got nothing else to send, so stop listening for G_IO_OUT. */
       if (connection->_priv->buffer == NULL)
 	{
-	  gnetwork_thread_source_remove (connection->_priv->source_id);
-	  connection->_priv->source_cond = (G_IO_IN | G_IO_PRI | G_IO_ERR | G_IO_HUP);
-	  connection->_priv->source_id =
-	    gnetwork_thread_io_add_watch (connection->_priv->channel,
-					  connection->_priv->source_cond,
-					  (GIOFunc) io_channel_handler, connection);
+	  if (connection->_priv->source_id != 0)
+	     gnetwork_thread_source_remove (connection->_priv->source_id);
+
+	  if (connection->_priv->channel != NULL)
+	    {
+	      connection->_priv->source_cond = (G_IO_IN | G_IO_PRI | G_IO_ERR | G_IO_HUP);
+	      connection->_priv->source_id =
+		gnetwork_thread_io_add_watch (connection->_priv->channel,
+					      connection->_priv->source_cond,
+					      (GIOFunc) io_channel_handler, connection);
+	    }
+
 	  retval = FALSE;
 	}
     }
@@ -447,6 +452,8 @@ io_channel_handler (GIOChannel * channel, GIOCondition cond, GNetworkTcpConnecti
 }
 
 
+#ifdef _USE_SSL
+
 static void
 open_ssl_connection (GNetworkTcpConnection * connection)
 {
@@ -505,10 +512,9 @@ open_ssl_connection (GNetworkTcpConnection * connection)
     }
 
   connection->_priv->source_cond = (G_IO_IN | G_IO_PRI | G_IO_ERR | G_IO_HUP);
-  connection->_priv->source_id = gnetwork_thread_io_add_watch (connection->_priv->channel,
-							       connection->_priv->source_cond,
-							       (GIOFunc) io_channel_handler,
-							       connection);
+  connection->_priv->source_id =
+    gnetwork_thread_io_add_watch (connection->_priv->channel, connection->_priv->source_cond,
+				  (GIOFunc) io_channel_handler, connection);
 
   connection->_priv->tcp_status = GNETWORK_TCP_CONNECTION_OPEN;
   connection->_priv->cxn_status = GNETWORK_CONNECTION_OPEN;
@@ -519,6 +525,8 @@ open_ssl_connection (GNetworkTcpConnection * connection)
   g_object_thaw_notify (G_OBJECT (connection));
 }
 
+#endif /* _USE_SSL */
+
 
 static void
 proxy_done_cb (GIOChannel * channel, const GError * error, GNetworkTcpConnection * connection)
@@ -541,17 +549,18 @@ proxy_done_cb (GIOChannel * channel, const GError * error, GNetworkTcpConnection
     }
 
   /* We still need SSL. */
+#ifdef _USE_SSL
   if (connection->_priv->ssl_enabled)
     {
       open_ssl_connection (connection);
     }
   else
+#endif /* _USE_SSL */
     {
       connection->_priv->source_cond = (G_IO_IN | G_IO_PRI | G_IO_ERR | G_IO_HUP);
-      connection->_priv->source_id = gnetwork_thread_io_add_watch (connection->_priv->channel,
-								   connection->_priv->source_cond,
-								   (GIOFunc) io_channel_handler,
-								   connection);
+      connection->_priv->source_id =
+	gnetwork_thread_io_add_watch (connection->_priv->channel, connection->_priv->source_cond,
+				      (GIOFunc) io_channel_handler, connection);
 
       connection->_priv->tcp_status = GNETWORK_TCP_CONNECTION_OPEN;
       connection->_priv->cxn_status = GNETWORK_CONNECTION_OPEN;
@@ -605,10 +614,12 @@ connect_suceeded (GNetworkTcpConnection * connection)
     {
       open_proxy_connection (connection);
     }
+#ifdef _USE_SSL
   else if (connection->_priv->ssl_enabled)
     {
       open_ssl_connection (connection);
     }
+#endif /* _USE_SSL */
   /* We're good to go. */
   else
     {
@@ -621,10 +632,9 @@ connect_suceeded (GNetworkTcpConnection * connection)
       g_object_thaw_notify (G_OBJECT (connection));
 
       connection->_priv->source_cond = (G_IO_IN | G_IO_PRI | G_IO_ERR | G_IO_HUP);
-      connection->_priv->source_id = gnetwork_thread_io_add_watch (connection->_priv->channel,
-								   connection->_priv->source_cond,
-								   (GIOFunc) io_channel_handler,
-								   connection);
+      connection->_priv->source_id =
+	gnetwork_thread_io_add_watch (connection->_priv->channel, connection->_priv->source_cond,
+				      (GIOFunc) io_channel_handler, connection);
     }
 }
 
@@ -779,7 +789,8 @@ open_client_connection (GNetworkTcpConnection * connection)
 
 	  connection->_priv->source_cond = GNETWORK_IO_ANY;
 	  connection->_priv->source_id =
-	    gnetwork_thread_io_add_watch (connection->_priv->channel, GNETWORK_IO_ANY,
+	    gnetwork_thread_io_add_watch (connection->_priv->channel,
+					  connection->_priv->source_cond,
 					  (GIOFunc) connect_done_handler, connection);
 	  return;
 	}
@@ -1015,10 +1026,10 @@ gnetwork_tcp_connection_open (GNetworkTcpConnection * connection)
       else
 #endif /* _USE_SSL */
 	{
-
+	  connection->_priv->source_cond = (G_IO_IN | G_IO_PRI | G_IO_ERR | G_IO_HUP);
 	  connection->_priv->source_id =
 	    gnetwork_thread_io_add_watch (connection->_priv->channel,
-					  (G_IO_IN | G_IO_PRI | G_IO_ERR | G_IO_HUP),
+					  connection->_priv->source_cond,
 					  (GIOFunc) io_channel_handler, connection);
 
 	  connection->_priv->tcp_status = GNETWORK_TCP_CONNECTION_OPEN;
@@ -1279,7 +1290,7 @@ gnetwork_tcp_connection_dispose (GObject * object)
   GNetworkTcpConnection *connection = GNETWORK_TCP_CONNECTION (object);
 
   if (connection->_priv->tcp_status > GNETWORK_TCP_CONNECTION_CLOSED)
-    gnetwork_tcp_connection_close (connection);
+    gnetwork_connection_close (GNETWORK_CONNECTION (connection));
 
   if (G_OBJECT_CLASS (parent_class)->dispose)
     (*G_OBJECT_CLASS (parent_class)->dispose) (object);
@@ -1308,30 +1319,6 @@ gnetwork_tcp_connection_finalize (GObject * object)
  *  GType Functions  *
  * ***************** */
 
-static void
-gnetwork_tcp_connection_base_init (gpointer class)
-{
-#if 0
-#ifdef _USE_SSL
-  _gnetwork_ssl_initialize ();
-#endif /* _USE_SSL */
-#endif /* 0 */
-  _gnetwork_tcp_proxy_initialize ();
-}
-
-
-static void
-gnetwork_tcp_connection_base_finalize (gpointer class)
-{
-#if 0
-#ifdef _USE_SSL
-  _gnetwork_ssl_shutdown ();
-#endif /* _USE_SSL */
-#endif /* 0 */
-  _gnetwork_tcp_proxy_shutdown ();
-}
-
-
 static void
 gnetwork_tcp_connection_class_init (GNetworkTcpConnectionClass * class)
 {
@@ -1357,23 +1344,19 @@ gnetwork_tcp_connection_class_init (GNetworkTcpConnectionClass * class)
 						      _("The type of proxy to use, depending on "
 							"the protocol."),
 						      GNETWORK_TYPE_TCP_PROXY_TYPE,
-						      GNETWORK_TCP_PROXY_HTTP,
-						      (G_PARAM_READWRITE | G_PARAM_CONSTRUCT)));
+						      GNETWORK_TCP_PROXY_HTTP, G_PARAM_READWRITE));
   g_object_class_install_property (object_class, ADDRESS,
 				   g_param_spec_string ("address", _("Address"),
 							_("The hostname or IP address to connect "
-							  "to."),
-							NULL,
-							(G_PARAM_READWRITE | G_PARAM_CONSTRUCT)));
+							  "to."), NULL, G_PARAM_READWRITE));
   g_object_class_install_property (object_class, IP_ADDRESS,
 				   g_param_spec_boxed ("ip-address", _("IP Address"),
-							_("The IP address to connect to."),
+						       _("The IP address to connect to."),
 						       GNETWORK_TYPE_IP_ADDRESS, G_PARAM_READABLE));
   g_object_class_install_property (object_class, PORT,
 				   g_param_spec_uint ("port", _("Port Number"),
 						      _("The port number to connect to."),
-						      0, 65535, 0,
-						      (G_PARAM_READWRITE | G_PARAM_CONSTRUCT)));
+						      0, 65535, 0, G_PARAM_READWRITE));
   g_object_class_install_property (object_class, LOCAL_ADDRESS,
 				   g_param_spec_boxed ("local-address", _("Local Address"),
 						       _("The IP address of this computer."),
@@ -1385,8 +1368,7 @@ gnetwork_tcp_connection_class_init (GNetworkTcpConnectionClass * class)
 							"through."),
 						      0, 65535, 0, G_PARAM_READABLE));
   g_object_class_install_property (object_class, SOCKET_FD,
-				   g_param_spec_pointer ("socket",
-							 _("Socket File Descriptor"),
+				   g_param_spec_pointer ("socket", _("Socket File Descriptor"),
 							 _("The socket file descriptor."),
 							 (G_PARAM_PRIVATE | G_PARAM_READWRITE)));
 
@@ -1395,7 +1377,7 @@ gnetwork_tcp_connection_class_init (GNetworkTcpConnectionClass * class)
 				   g_param_spec_boolean ("ssl-enabled", _("SSL Enabled"),
 							 _("Whether or not SSL will be used with "
 							   "this connection."), FALSE,
-							 (G_PARAM_READWRITE | G_PARAM_CONSTRUCT)));
+							 G_PARAM_READWRITE));
   g_object_class_install_property (object_class, SSL_AUTH_TYPE,
 				   g_param_spec_enum ("authentication-type",
 						      _("Authentication Type"),
@@ -1403,25 +1385,24 @@ gnetwork_tcp_connection_class_init (GNetworkTcpConnectionClass * class)
 							"performed with this connection."),
 						      GNETWORK_TYPE_SSL_AUTH_TYPE,
 						      GNETWORK_SSL_AUTH_ANONYMOUS,
-						      (G_PARAM_READWRITE | G_PARAM_CONSTRUCT)));
+						      G_PARAM_READWRITE));
   g_object_class_install_property (object_class, SSL_CA_FILE,
 				   g_param_spec_string ("authority-file",
 							_("Certificate Authorities File"),
 							_("The path to a file which contains the "
 							  "X.509 certificates of trusted signers."),
 							NULL,
-							(G_PARAM_READWRITE | G_PARAM_CONSTRUCT)));
+							G_PARAM_READWRITE));
   g_object_class_install_property (object_class, SSL_CERT_FILE,
 				   g_param_spec_string ("certificate-file", _("Certificate File"),
 							_("The path to a file which contains X.509 "
 							  "certificates to distribute to clients."),
-							NULL,
-							(G_PARAM_READWRITE | G_PARAM_CONSTRUCT)));
+							NULL, G_PARAM_READWRITE));
   g_object_class_install_property (object_class, SSL_KEY_FILE,
 				   g_param_spec_string ("key-file", _("Key File"),
 							_("The path to a file which contains X.509 "
 							  "keys to distribute to clients."), NULL,
-							(G_PARAM_READWRITE | G_PARAM_CONSTRUCT)));
+							G_PARAM_READWRITE));
 
   /* GNetworkConnectionIface Properties */
   g_object_class_override_property (object_class, CXN_TYPE, "connection-type");
@@ -1438,6 +1419,13 @@ gnetwork_tcp_connection_class_init (GNetworkTcpConnectionClass * class)
 		  G_STRUCT_OFFSET (GNetworkTcpConnectionClass, certificate_error),
 		  NULL, NULL, _gnetwork_marshal_VOID__BOXED_FLAGS,
 		  G_TYPE_NONE, 2, GNETWORK_TYPE_SSL_CERT, GNETWORK_TYPE_SSL_CERT_ERROR_FLAGS);
+
+#if 0
+#ifdef _USE_SSL
+  _gnetwork_ssl_initialize ();
+#endif /* _USE_SSL */
+#endif /* 0 */
+  _gnetwork_tcp_proxy_initialize ();
 }
 
 
@@ -1459,10 +1447,17 @@ gnetwork_tcp_connection_instance_init (GNetworkTcpConnection * connection)
   gnetwork_ip_address_init (&(connection->_priv->local_address));
   gnetwork_ip_address_init (&(connection->_priv->proxy_ip_address));
 
+  connection->_priv->ssl_enabled = FALSE;
+  connection->_priv->auth_type = GNETWORK_SSL_AUTH_INVALID;
   connection->_priv->ca_file = NULL;
   connection->_priv->cert_file = NULL;
   connection->_priv->key_file = NULL;
 
+  connection->_priv->buffer = NULL;
+  connection->_priv->bytes_received = 0;
+  connection->_priv->bytes_sent = 0;
+  connection->_priv->tcp_proxy_type = GNETWORK_TCP_PROXY_NONE;
+
   connection->_priv->channel = NULL;
   connection->_priv->source_id = 0;
   connection->_priv->source_cond = 0;
@@ -1483,8 +1478,8 @@ gnetwork_tcp_connection_get_type (void)
     {
       static const GTypeInfo info = {
 	sizeof (GNetworkTcpConnectionClass),
-	(GBaseInitFunc) gnetwork_tcp_connection_base_init,
-	(GBaseFinalizeFunc) gnetwork_tcp_connection_base_finalize,
+	(GBaseInitFunc) NULL,
+	(GBaseFinalizeFunc) NULL,
 	(GClassInitFunc) gnetwork_tcp_connection_class_init,
 	(GClassFinalizeFunc) NULL,
 	NULL,			/* class data */
