@@ -463,6 +463,22 @@ gnetwork_tcp_server_open (GNetworkTcpServer * server)
       return;
     }
 
+  if (server->_priv->port == 0)
+    {
+      socklen_t socklen = sizeof sa;
+
+      if (getsockname (server->_priv->sockfd, (struct sockaddr *) &sa, &socklen) == 0)
+       {
+         if (sa.ss_family == AF_INET)
+           server->_priv->port = g_ntohs (((struct sockaddr_in *) &sa)->sin_port);
+         else if (sa.ss_family == AF_INET6)
+           server->_priv->port = g_ntohs (((struct sockaddr_in6 *) &sa)->sin6_port);
+
+         if (server->_priv->port != 0)
+           g_object_notify (G_OBJECT (server), "port");
+       }
+    }
+
   server->_priv->status = GNETWORK_SERVER_OPEN;
   g_object_notify (G_OBJECT (server), "status");
 
