@@ -707,6 +707,22 @@ gnetwork_udp_datagram_open (GNetworkUdpDatagram * udp)
 	return;
     }
 
+  if (udp->_priv->port == 0)
+    {
+      socklen_t socklen = sizeof sa;
+
+      if (getsockname (udp->_priv->sockfd, (struct sockaddr *) &sa, &socklen) == 0)
+       {
+         if (sa.ss_family == AF_INET)
+           udp->_priv->port = g_ntohs (((struct sockaddr_in *) &sa)->sin_port);
+         else if (sa.ss_family == AF_INET6)
+           udp->_priv->port = g_ntohs (((struct sockaddr_in6 *) &sa)->sin6_port);
+
+         if (udp->_priv->port != 0)
+           g_object_notify (G_OBJECT (udp), "port");
+       }
+    }
+
   udp->_priv->channel = g_io_channel_unix_new (udp->_priv->sockfd);
   g_io_channel_set_encoding (udp->_priv->channel, NULL, NULL);
   g_io_channel_set_buffered (udp->_priv->channel, FALSE);
